import 'package:after_layout/after_layout.dart';
import 'package:flutter/material.dart';

import 'navigation_presenter_interface.dart';

typedef Widget ScreenBuilder<T>(ValueKey<T> key, BuildContext ctx);

class OsamNavigator<T> extends StatefulWidget {
  final Map<T, ScreenBuilder<T>> screens;
  final INavigation<T> presenter;
  final bool rootNavigator;

  OsamNavigator({Key key, this.screens, this.presenter, this.rootNavigator = false})
      : super(key: key);

  @override
  _OsamNavigatorState createState() => _OsamNavigatorState<T>();
}

class _OsamNavigatorState<T> extends State<OsamNavigator<T>> with AfterLayoutMixin {
  final List<T> _routes = [];
  final RouteObserver<PageRoute> routeObserver = RouteObserver<PageRoute>();

  Map<T, ScreenBuilder> get screens => widget.screens;

  INavigation<T> get presenter => widget.presenter;

  @override
  Widget build(BuildContext context) => Container();

  @override
  void afterFirstLayout(BuildContext context) {
    final navigator = Navigator.of(context, rootNavigator: widget.rootNavigator);
    final initialRoutes = presenter.routeStream.initialData;
    _routes.addAll(initialRoutes);
    if (initialRoutes.isNotEmpty) {
      final firstRoute = initialRoutes.first;
      navigator.pushReplacement(MaterialPageRoute(
          builder: ((ctx) => screens[firstRoute](ValueKey(firstRoute), ctx)),
          settings: RouteSettings(isInitialRoute: true)));
      final others = initialRoutes.sublist(1, initialRoutes.length);
      others.forEach((route) {
        navigator.push(MaterialPageRoute(builder: ((ctx) {
          return screens[route](ValueKey(route), ctx);
        })));
      });
    }

    presenter.routeStream.stream.listen((routes) async {
      final routesDifferentNumber = routes.length - _routes.length;
      if (routesDifferentNumber == 0) {
        navigator.pushReplacement(MaterialPageRoute(builder: ((ctx) {
          return screens[routes.last](ValueKey(routes.last), ctx);
        })));
      } else if (routesDifferentNumber > 0) {
        final pushRoutes = routes.sublist(_routes.length, routes.length);
        if (routesDifferentNumber == 1 && _routes.isEmpty) {
          navigator.pushReplacement(MaterialPageRoute(
            builder: ((ctx) => screens[routes.first](ValueKey(routes.first), ctx)),
          ));
        } else {
          pushRoutes.forEach((route) async {
            navigator.push(MaterialPageRoute(builder: ((ctx) {
              return screens[route](ValueKey(route), ctx);
            })));
          });
        }
      } else if (routesDifferentNumber < 0 && _routes.isNotEmpty) {
        if (_routes.first != routes.first) {
          navigator.pushAndRemoveUntil(MaterialPageRoute(builder: ((ctx) {
            return screens[routes.first](ValueKey(routes.first), ctx);
          })), (route) => false);
        } else {
          final removedRoutes = _routes.sublist(routes.length, _routes.length);
          removedRoutes.forEach((route) async {
            navigator.pop();
          });
        }
      }
      _routes.clear();
      _routes.addAll(routes);
    });
  }
}

//
//
//typedef void RouteTo<T>(T tab);
//
//class TabNavigator<T> extends StatelessWidget {
//  final List<Widget> screens;
//  final List<T> enumTabs;
//  final List<Widget> tabs;
//  final _navigators = <T, GlobalKey<NavigatorState>>{};
//  final _screenKeys = <GlobalKey>[];
//  final T initialTab;
//  final Stream<T> tabStream;
//  final RouteTo<T> routeTo;
//  final Color bottomBackground;
//
//  TabNavigator(
//      {Key key,
//        @required this.screens,
//        @required this.tabs,
//        @required this.enumTabs,
//        @required this.initialTab,
//        @required this.tabStream,
//        @required this.routeTo,
//        @required this.bottomBackground})
//      : super(key: key) {
//    this
//        ._navigators
//        .addEntries(enumTabs.map((tab) => MapEntry(tab, GlobalKey<NavigatorState>())).toList());
//    this._screenKeys.addAll(screens.map((screen) => GlobalKey()).toList());
//  }
//
//  @override
//  Widget build(BuildContext context) {

//    return StreamBuilder(
//        stream: tabStream,
//        initialData: initialTab,
//        builder: (context, AsyncSnapshot<T> snapshot) {
//          final currentTab = snapshot.data;
//          INavigation currentPresenter;
//          if (currentTab == RouteTab.search) {
//            currentPresenter = searchPresenter;
//          } else if (currentTab == RouteTab.settings) {
//            currentPresenter = settingsPresenter;
//          }
//          return WillPopScope(
//            onWillPop: () async {
//              if (currentPresenter.initialRoutes.length > 1) {
//                currentPresenter.pop();
//                return false;
//              } else {
//                return true;
//              }
//            },
//            child: Stack(
//              children: <Widget>[
//                IndexedStack(
//                  index: enumTabs.indexOf(currentTab),
//                  children: screens
//                      .map((screen) => _NavigatorPage(
//                    key: _screenKeys[screens.indexOf(screen)],
//                    navKey: _navigators[screens.indexOf(screen)],
//                    child: screen,
//                  ))
//                      .toList(),
//                ),
//                Positioned(
//                  bottom: 0,
//                  child: Container(
//                    color: bottomBackground,
//                    padding: const EdgeInsets.symmetric(vertical: 5),
//                    constraints:
//                    BoxConstraints(maxWidth: MediaQuery.of(context).size.width, minHeight: 48),
//                    child: Row(
//                      children: enumTabs
//                          .map((enumTab) => Expanded(
//                          child: GestureDetector(
//                            child: Container(
//                              alignment: Alignment.center,
//                              color: Colors.transparent,
//                              child: tabs[enumTabs.indexOf(enumTab)],
//                            ),
//                            onTap: () {
//                              if (currentTab != enumTab) {
//                                routeTo(enumTab);
//                              } else {
//                                final routes = currentPresenter.initialRoutes;
//                                if (routes.length > 1) {
//                                  routes.sublist(1, routes.length).forEach((route) {
//                                    currentPresenter.pop();
//                                  });
//                                }
//                              }
//                            },
//                          )))
//                          .toList(),
//                    ),
//                  ),
//                )
//              ],
//            ),
//          );
//        });
//  }
//}
//
//class _NavigatorPage extends StatelessWidget {
//  final GlobalKey<NavigatorState> navKey;
//  final Widget child;
//
//  _NavigatorPage({Key key, this.navKey, this.child});
//
//  @override
//  Widget build(BuildContext context) {
//    return Navigator(
//      key: navKey,
//      onGenerateRoute: (RouteSettings settings) {
//        return CupertinoPageRoute(
//          settings: RouteSettings(isInitialRoute: true),
//          builder: (BuildContext context) => child,
//        );
//      },
//    );
//  }
//}
